@@ -14,6 +14,8 @@ def parse_file(path: Path) -> Optional[List[dt.AnnotationFile]]:
         reader = csv.reader(f)
         for row in reader:
             filename, *tags = map(lambda s: s.strip(), row)
+            if filename == "":
+                continue
             annotations = [dt.make_tag(tag) for tag in tags if len(tag) > 0]
             annotation_classes = set([annotation.annotation_class for annotation in annotations])
             files.append(dt.AnnotationFile(path, filename, annotation_classes, annotations))
