@@ -1,155 +1,80 @@
-import multiprocessing as mp
-from pathlib import Path
-from typing import Collection, List, Optional, Union
+from typing import List, Optional
 
 import numpy as np
 
+from darwin.cli_functions import _error, _load_client
 from darwin.dataset import LocalDataset
-from darwin.torch.transforms import Compose, ConvertPolygonsToInstanceMasks, ConvertPolygonsToSegmentationMask
-from darwin.torch.utils import convert_polygons_to_sequences, load_pil_image, polygon_area
+from darwin.dataset.identifier import DatasetIdentifier
+from darwin.dataset.utils import load_pil_image
+from darwin.torch.transforms import Compose, ConvertPolygonsToInstanceMasks, ConvertPolygonsToSemanticMask
+from darwin.torch.utils import polygon_area
+from darwin.utils import convert_polygons_to_sequences
 
 
 def get_dataset(
-    dataset_path: Union[Path, str],
+    dataset_slug: str,
     dataset_type: str,
     partition: Optional[str] = None,
     split: str = "default",
     split_type: str = "random",
-    release_name: Optional[str] = None,
     transform: Optional[List] = None,
 ):
     """ Creates and returns a dataset
 
     Parameters
     ----------
-    dataset_path: Path, str
-        Path to the location of the dataset on the file system
+    dataset_slug: str
+        Slug of the dataset to retrieve
     dataset_type: str
-        The type of dataset [classification, instance_segmentation, semantic_segmentation]
+        The type of dataset [classification, instance-segmentation, semantic-segmentation]
     partition: str
-        Selects one of the partitions [train, val, test]
+        Selects one of the partitions [train, val, test, None]. (Default: None)
     split: str
-        Selects the split that defines the percentages used (use 'default' to select the default split)
+        Selects the split that defines the percentages used. (Default: 'default')
     split_type: str
-        Heuristic used to do the split [random, stratified]
-    release_name: str
-        Version of the dataset
+        Heuristic used to do the split [random, stratified]. (Default: 'random')
     transform : list[torchvision.transforms]
         List of PyTorch transforms
     """
     dataset_functions = {
         "classification": ClassificationDataset,
-        "instance_segmentation": InstanceSegmentationDataset,
-        "semantic_segmentation": SemanticSegmentationDataset,
+        "instance-segmentation": InstanceSegmentationDataset,
+        "semantic-segmentation": SemanticSegmentationDataset,
     }
     dataset_function = dataset_functions.get(dataset_type)
     if not dataset_function:
         list_of_types = ", ".join(dataset_functions.keys())
-        raise ValueError(f"dataset_type needs to be one of '{list_of_types}'")
+        _error(f"dataset_type needs to be one of '{list_of_types}'")
 
-    if isinstance(dataset_path, str):
-        dataset_path = Path(dataset_path)
+    identifier = DatasetIdentifier.parse(dataset_slug)
+    client = _load_client(offline=True)
 
-    return dataset_function(
-        dataset_path=dataset_path,
-        partition=partition,
-        split=split,
-        split_type=split_type,
-        release_name=release_name,
-        transform=transform,
+    for p in client.list_local_datasets(team=identifier.team_slug):
+        if identifier.dataset_slug == p.name:
+            return dataset_function(
+                dataset_path=p,
+                partition=partition,
+                split=split,
+                split_type=split_type,
+                release_name=identifier.version,
+                transform=transform,
+            )
+
+    for p in client.list_deprecated_local_datasets(team=identifier.team_slug):
+        if identifier.dataset_slug == p.name:
+            _error(
+                f"Found a local version of the dataset {identifier.dataset_slug} which uses a deprecated format. "
+                f"Run `darwin dataset migrate {identifier}` if you want to be able to use it in darwin-py."
+            )
+
+    _error(
+        f"Dataset '{identifier.dataset_slug}' does not exist locally. "
+        f"Use 'darwin dataset remote' to see all the available datasets, "
+        f"and 'darwin dataset pull' to pull them."
     )
 
 
-class DarwinDataset(LocalDataset):
-    def measure_mean_std(self, multi_threaded: bool = True):
-        """Computes mean and std of train images, given the train loader
-
-        Parameters
-        ----------
-        multi_threaded : bool
-            Uses multiprocessing to download the dataset in parallel.
-
-        Returns
-        -------
-        mean : ndarray[double]
-            Mean value (for each channel) of all pixels of the images in the input folder
-        std : ndarray[double]
-            Standard deviation (for each channel) of all pixels of the images in the input folder
-        """
-        if multi_threaded:
-            # Set up a pool of workers
-            with mp.Pool(mp.cpu_count()) as pool:
-                # Online mean
-                results = pool.map(self._return_mean, self.images_path)
-                mean = np.sum(np.array(results), axis=0) / len(self.images_path)
-                # Online image_classification deviation
-                results = pool.starmap(self._return_std, [[item, mean] for item in self.images_path])
-                std_sum = np.sum(np.array([item[0] for item in results]), axis=0)
-                total_pixel_count = np.sum(np.array([item[1] for item in results]))
-                std = np.sqrt(std_sum / total_pixel_count)
-                # Shut down the pool
-                pool.close()
-                pool.join()
-            return mean, std
-        else:
-            # Online mean
-            results = [self._return_mean(f) for f in self.images_path]
-            mean = np.sum(np.array(results), axis=0) / len(self.images_path)
-            # Online image_classification deviation
-            results = [self._return_std(f, mean) for f in self.images_path]
-            std_sum = np.sum(np.array([item[0] for item in results]), axis=0)
-            total_pixel_count = np.sum(np.array([item[1] for item in results]))
-            std = np.sqrt(std_sum / total_pixel_count)
-            return mean, std
-
-    def measure_weights(self, **kwargs):
-        """Computes the class balancing weights (not the frequencies!!) given the train loader
-
-        Returns
-        -------
-        class_weights : ndarray[double]
-            Weight for each class in the train set (one for each class)
-        """
-        raise NotImplementedError("Base class Dataset does not have an implementation for this")
-
-    @staticmethod
-    def _compute_weights(labels: Collection):
-        """Given an array of labels computes the weights normalized
-
-        Parameters
-        ----------
-        labels : ndarray[int]
-            Array of labels
-
-        Returns
-        -------
-        ndarray[float]
-            Array of weights (one for each unique class) which are the inverse of their frequency
-        """
-        class_support = np.unique(labels, return_counts=True)[1]
-        class_frequencies = class_support / len(labels)
-        # Class weights are the inverse of the class frequencies
-        class_weights = 1 / class_frequencies
-        # Normalize vector to sum up to 1.0 (in case the Loss function does not do it)
-        class_weights /= class_weights.sum()
-        return class_weights
-
-    # Loads an image with OpenCV and returns the channel wise means of the image.
-    @staticmethod
-    def _return_mean(image_path):
-        img = np.array(load_pil_image(image_path))
-        mean = np.array([np.mean(img[:, :, 0]), np.mean(img[:, :, 1]), np.mean(img[:, :, 2])])
-        return mean / 255.0
-
-    # Loads an image with OpenCV and returns the channel wise std of the image.
-    @staticmethod
-    def _return_std(image_path, mean):
-        img = np.array(load_pil_image(image_path)) / 255.0
-        m2 = np.square(np.array([img[:, :, 0] - mean[0], img[:, :, 1] - mean[1], img[:, :, 2] - mean[2]]))
-        return np.sum(np.sum(m2, axis=1), 1), m2.size / 3.0
-
-
-class ClassificationDataset(DarwinDataset):
+class ClassificationDataset(LocalDataset):
     def __init__(self, transform: Optional[List] = None, **kwargs):
         """See class `LocalDataset` for documentation"""
         super().__init__(annotation_type="tag", **kwargs)
@@ -166,10 +91,10 @@ class ClassificationDataset(DarwinDataset):
         The return value is a dict with the following fields:
             image_id: int
                 The index of the image in the split
-            original_filename: str
+            image_path: str
                 The path to the image on the file system
             category_id : int
-                The single label of the image selected.
+                The single label of the image selected
         """
         img = load_pil_image(self.images_path[index])
         if self.transform is not None:
@@ -179,9 +104,7 @@ class ClassificationDataset(DarwinDataset):
         annotations = target.pop("annotations")
         tags = [self.classes.index(a["name"]) for a in annotations if "tag" in a]
         if len(tags) > 1:
-            raise ValueError(
-                f"Multiple tags defined for this image ({tags}). " f"This is not valid in a classification dataset."
-            )
+            raise ValueError(f"Multiple tags defined for this image ({tags}). This is not supported at the moment.")
         if len(tags) == 0:
             raise ValueError(
                 f"No tags defined for this image ({self.annotations_path[index]})."
@@ -209,8 +132,8 @@ class ClassificationDataset(DarwinDataset):
         return self._compute_weights(labels)
 
 
-class InstanceSegmentationDataset(DarwinDataset):
-    def __init__(self, transform: Optional[List] = None, convert_polygons_to_masks: Optional[bool] = True, **kwargs):
+class InstanceSegmentationDataset(LocalDataset):
+    def __init__(self, transform: Optional[List] = None, **kwargs):
         """See `LocalDataset` class for documentation"""
         super().__init__(annotation_type="polygon", **kwargs)
 
@@ -218,28 +141,25 @@ class InstanceSegmentationDataset(DarwinDataset):
         if self.transform is not None and isinstance(self.transform, list):
             self.transform = Compose(self.transform)
 
-        self.convert_polygons = ConvertPolygonsToInstanceMasks() if convert_polygons_to_masks else None
+        self.convert_polygons = ConvertPolygonsToInstanceMasks()
 
     def __getitem__(self, index: int):
         """
-
         Notes
         -----
         The return value is a dict with the following fields:
             image_id : int
                 Index of the image inside the dataset
-            original_filename: str
+            image_path: str
                 The path to the image on the file system
-            annotations : list[Dict]
-                List of annotations, where each annotation is a dict with:
-                category_id : int
-                    The single label of the image selected.
-                segmentation : ndarray(1,)
-                    Array of points [x,y,x,y,x,y ...] composing the polygon enclosing the object
-                bbox : ndarray(1,)
-                    Coordinates of the bounding box enclosing the instance as [x, y, w, h]
-                area : float
-                    Area of the polygon
+            labels : tensor(n)
+                The class label of each one of the instances
+            masks : tensor(n, H, W)
+                Segmentation mask of each one of the instances
+            boxes : tensor(n, 4)
+                Coordinates of the bounding box enclosing the instances as [x, y, x, y]
+            area : float
+                Area in pixels of each one of the instances
         """
         img = load_pil_image(self.images_path[index])
         target = self.parse_json(index)
@@ -250,7 +170,9 @@ class InstanceSegmentationDataset(DarwinDataset):
                 print(f"Warning: missing polygon in annotation {self.annotations_path[index]}")
             # Extract the sequences of coordinates from the polygon annotation
             annotation_type = "polygon" if "polygon" in annotation else "complex_polygon"
-            sequences = convert_polygons_to_sequences(annotation[annotation_type]["path"])
+            sequences = convert_polygons_to_sequences(
+                annotation[annotation_type]["path"], height=target["height"], width=target["width"],
+            )
             # Compute the bbox of the polygon
             x_coords = [s[0::2] for s in sequences]
             y_coords = [s[1::2] for s in sequences]
@@ -275,8 +197,7 @@ class InstanceSegmentationDataset(DarwinDataset):
             )
         target["annotations"] = annotations
 
-        if self.convert_polygons is not None:
-            img, target = self.convert_polygons(img, target)
+        img, target = self.convert_polygons(img, target)
         if self.transform is not None:
             img, target = self.transform(img, target)
 
@@ -300,8 +221,8 @@ class InstanceSegmentationDataset(DarwinDataset):
         return self._compute_weights(labels)
 
 
-class SemanticSegmentationDataset(DarwinDataset):
-    def __init__(self, transform: Optional[List] = None, convert_polygons_to_masks: Optional[bool] = True, **kwargs):
+class SemanticSegmentationDataset(LocalDataset):
+    def __init__(self, transform: Optional[List] = None, **kwargs):
         """See `LocalDataset` class for documentation"""
         super().__init__(annotation_type="polygon", **kwargs)
 
@@ -309,7 +230,7 @@ class SemanticSegmentationDataset(DarwinDataset):
         if self.transform is not None and isinstance(self.transform, list):
             self.transform = Compose(self.transform)
 
-        self.convert_polygons = ConvertPolygonsToSegmentationMask() if convert_polygons_to_masks else None
+        self.convert_polygons = ConvertPolygonsToSemanticMask()
 
     def __getitem__(self, index: int):
         """See superclass for documentation
@@ -319,28 +240,27 @@ class SemanticSegmentationDataset(DarwinDataset):
         The return value is a dict with the following fields:
             image_id : int
                 Index of the image inside the dataset
-            original_filename: str
+            image_path: str
                 The path to the image on the file system
-            annotations : list
-                List of annotations, where each annotation is a dict with:
-                category_id : TODO complete documentation
-                segmentation :
+            mask : tensor(H, W)
+                Segmentation mask where each pixel encodes a class label
         """
         img = load_pil_image(self.images_path[index])
         target = self.parse_json(index)
 
         annotations = []
         for obj in target["annotations"]:
-            sequences = convert_polygons_to_sequences(obj["polygon"]["path"])
+            sequences = convert_polygons_to_sequences(
+                obj["polygon"]["path"], height=target["height"], width=target["width"],
+            )
             # Discard polygons with less than three points
             sequences[:] = [s for s in sequences if len(s) >= 6]
             if not sequences:
                 continue
-            annotations.append({"category_id": self.classes.index(obj["name"]), "segmentation": np.array(sequences)})
+            annotations.append({"category_id": self.classes.index(obj["name"]), "segmentation": sequences})
         target["annotations"] = annotations
 
-        if self.convert_polygons is not None:
-            img, target = self.convert_polygons(img, target)
+        img, target = self.convert_polygons(img, target)
         if self.transform is not None:
             img, target = self.transform(img, target)
 
