@@ -8,9 +8,10 @@ from upolygon import draw_polygon
 
 import darwin.datatypes as dt
 from darwin.config import Config
+from darwin.exceptions import OutdatedDarwinJSONFormat, UnsupportedFileType
 
 SUPPORTED_IMAGE_EXTENSIONS = [".png", ".jpeg", ".jpg", ".jfif", ".tif", ".bmp"]
-SUPPORTED_VIDEO_EXTENSIONS = [".bpm", ".mov", ".mp4"]
+SUPPORTED_VIDEO_EXTENSIONS = [".avi", ".bpm", ".dcm", ".mov", ".mp4"]
 SUPPORTED_EXTENSIONS = SUPPORTED_IMAGE_EXTENSIONS + SUPPORTED_VIDEO_EXTENSIONS
 
 
@@ -128,6 +129,8 @@ def find_files(
             found_files.extend([f for f in path.glob(pattern) if is_extension_allowed(f.suffix)])
         elif is_extension_allowed(path.suffix):
             found_files.append(path)
+        else:
+            raise UnsupportedFileType(path)
 
     # Filter the list and return it
     files_to_exclude = set(files_to_exclude)
@@ -168,34 +171,77 @@ def persist_client_configuration(
     return config
 
 
-def parse_darwin_json(path: Path, count: int):
+def get_local_filename(metadata: dict):
+    return metadata["filename"]
+
+
+def parse_darwin_json(path: Union[str, Path], count: int):
+    path = Path(path)
     with path.open() as f:
         data = json.load(f)
         if not data["annotations"]:
             return None
-        annotations = list(filter(None, map(parse_darwin_annotation, data["annotations"])))
-        annotation_classes = set([annotation.annotation_class for annotation in annotations])
+        if "fps" in data["image"] or "frame_count" in data["image"]:
+            return parse_darwin_video(path, data, count)
+        else:
+            return parse_darwin_image(path, data, count)
 
-        return dt.AnnotationFile(
-            path,
-            data["image"]["original_filename"],
-            annotation_classes,
-            annotations,
-            data["image"]["width"],
-            data["image"]["height"],
-            data["image"]["url"],
-            data["image"].get("workview_url"),
-            data["image"].get("seq", count),
-        )
+
+def parse_darwin_image(path, data, count):
+    annotations = list(filter(None, map(parse_darwin_annotation, data["annotations"])))
+    annotation_classes = set([annotation.annotation_class for annotation in annotations])
+
+    return dt.AnnotationFile(
+        path,
+        get_local_filename(data["image"]),
+        annotation_classes,
+        annotations,
+        False,
+        data["image"]["width"],
+        data["image"]["height"],
+        data["image"]["url"],
+        data["image"].get("workview_url"),
+        data["image"].get("seq", count),
+    )
+
+
+def parse_darwin_video(path, data, count):
+    annotations = list(filter(None, map(parse_darwin_video_annotation, data["annotations"])))
+    annotation_classes = set([annotation.annotation_class for annotation in annotations])
+
+    if "width" not in data["image"] or "height" not in data["image"]:
+        raise OutdatedDarwinJSONFormat("Missing width/height in video, please re-export")
+
+    return dt.AnnotationFile(
+        path,
+        get_local_filename(data["image"]),
+        annotation_classes,
+        annotations,
+        True,
+        data["image"]["width"],
+        data["image"]["height"],
+        data["image"]["url"],
+        data["image"].get("workview_url"),
+        data["image"].get("seq", count),
+        data["image"]["frame_urls"],
+    )
 
 
 def parse_darwin_annotation(annotation: dict):
     name = annotation["name"]
     main_annotation = None
     if "polygon" in annotation:
-        main_annotation = dt.make_polygon(name, annotation["polygon"]["path"])
+        if "additional_paths" in annotation["polygon"]:
+            paths = [annotation["polygon"]["path"]] + annotation["polygon"]["additional_paths"]
+            main_annotation = dt.make_complex_polygon(name, paths)
+        else:
+            main_annotation = dt.make_polygon(name, annotation["polygon"]["path"])
     elif "complex_polygon" in annotation:
-        main_annotation = dt.make_complex_polygon(name, annotation["complex_polygon"]["path"])
+        if "additional_paths" in annotation["complex_polygon"]:
+            paths = annotation["complex_polygon"]["path"] + annotation["complex_polygon"]["additional_paths"]
+            main_annotation = dt.make_complex_polygon(name, paths)
+        else:
+            main_annotation = dt.make_complex_polygon(name, annotation["complex_polygon"]["path"])
     elif "bounding_box" in annotation:
         bounding_box = annotation["bounding_box"]
         main_annotation = dt.make_bounding_box(
@@ -207,6 +253,13 @@ def parse_darwin_annotation(annotation: dict):
         main_annotation = dt.make_line(name, annotation["line"]["path"])
     elif "keypoint" in annotation:
         main_annotation = dt.make_keypoint(name, annotation["keypoint"]["x"], annotation["keypoint"]["y"])
+    elif "ellipse" in annotation:
+        main_annotation = dt.make_ellipse(name, annotation["ellipse"])
+    elif "cuboid" in annotation:
+        main_annotation = dt.make_cuboid(name, annotation["cuboid"])
+    # TODO
+    # elif "skeleton" in annotation:
+    #     main_annotation = dt.make_skeleton(name, annotation["skeleton"]["nodes"])
 
     if not main_annotation:
         print(f"[WARNING] Unsupported annotation type: '{annotation.keys()}'")
@@ -222,6 +275,43 @@ def parse_darwin_annotation(annotation: dict):
     return main_annotation
 
 
+def parse_darwin_video_annotation(annotation: dict):
+    name = annotation["name"]
+    frame_annotations = {}
+    keyframes = {}
+    for f, frame in annotation["frames"].items():
+        frame_annotations[int(f)] = parse_darwin_annotation({**frame, **{"name": name}})
+        keyframes[int(f)] = frame["keyframe"]
+    return dt.make_video_annotation(frame_annotations, keyframes, annotation["segments"], annotation["interpolated"])
+
+
+def split_video_annotation(annotation):
+    if not annotation.is_video:
+        raise AttributeError("this is not a video annotation")
+
+    frame_annotations = []
+    for i, frame_url in enumerate(annotation.frame_urls):
+        annotations = [a.frames[i] for a in annotation.annotations if i in a.frames]
+        annotation_classes = set([annotation.annotation_class for annotation in annotations])
+        filename = f"{Path(annotation.filename).stem}/{i:07d}.jpg"
+
+        frame_annotations.append(
+            dt.AnnotationFile(
+                annotation.path,
+                filename,
+                annotation_classes,
+                annotations,
+                False,
+                annotation.image_width,
+                annotation.image_height,
+                frame_url,
+                annotation.workview_url,
+                annotation.seq,
+            )
+        )
+    return frame_annotations
+
+
 def ispolygon(annotation):
     return annotation.annotation_type in ["polygon", "complex_polygon"]
 
